#include "stm32f4xx.h"

#include "System.h"
#include "FSMC_LowLevel.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_rcc.h"

#include "SSD1963.h"

// Color
uint16_t  _color;
// Clipping region control
uint16_t _clipRgn;
// Clipping region borders
uint16_t _clipLeft;
uint16_t _clipTop;
uint16_t _clipRight;
uint16_t _clipBottom;

// Active Page
uint8_t  _activePage;
// Visual Page
uint8_t  _visualPage;

// ssd1963 specific
uint8_t _gpioStatus = 0;

void LCD_SetArea(uint16_t start_x, uint16_t start_y, uint16_t end_x, uint16_t end_y);

static void LCD_Delay(vu32 nCount);
//static void LCD_DelayUs(vu32 nCount);

/*
static uint16_t HDP=799;
static uint16_t HT=928;
static uint16_t HPS=46;
static uint16_t LPS=15;
static uint8_t  HPW=48;

static uint16_t VDP=479;
static uint16_t VT=525;
static uint16_t VPS=16;
static uint16_t FPS=8;
static uint16_t VPW=16;
*/

//  LCD
void ResetLcd ( void )
{
	GPIO_InitTypeDef GPIO_InitStructure;

	//  Reset - PD2
	//        nCS - PD7
	//        nWR - PD5

	// Enable GPIOD clocks
	RCC_AHB1PeriphClockCmd ( RCC_AHB1Periph_GPIOD, ENABLE );

	// PD2, PD5, PD7
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2 | GPIO_Pin_5 | GPIO_Pin_7;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
	GPIO_Init ( GPIOD, &GPIO_InitStructure );

	// PD2 := PD5 := 1, PD7 := 1 - nRES := nCS := nWR := 1
	GPIO_SetBits ( GPIOD, GPIO_Pin_2 | GPIO_Pin_5 | GPIO_Pin_7 );

	// 
	delay_ms ( 5 );
	// nRES := 0
	GPIO_ResetBits ( GPIOD, GPIO_Pin_2 );
	delay_ms ( 15 );
	// nRES := 1
	GPIO_SetBits ( GPIOD, GPIO_Pin_2 );
	delay_ms ( 15 );
} // ResetLcd

/*********************************************************************
* Macros:  PMPWaitBusy()
* Overview: waits for PMP cycle end.
********************************************************************/
#define PMPWaitBusy()  Nop();

/*********************************************************************
* Function:  void ResetDevice()
* Overview: Resets LCD, initializes PMP
*			Initialize low level IO port for mcu,
*			initialize SSD1963 for PCLK,
*			HSYNC, VSYNC etc
* Note: Need to set the backlight intensity by SetBacklight(BYTE intensity)
*		in main()
********************************************************************/
void LCD_Initialize(void)
{
	ResetLcd ( );

	//  FSMC
	FSMC_Init ( );

	//Set MN(multipliers) of PLL, VCO = crystal freq * (N+1)
	//PLL freq = VCO/M with 250MHz < VCO < 800MHz
	//The max PLL freq is around 120MHz. To obtain 120MHz as the PLL freq

	FSMC_LcdWriteCmd (0xE2);				// Set PLL with OSC = 10MHz (hardware)
//	Clr_Cs;
	FSMC_LcdWriteData(0x23);
	FSMC_LcdWriteData(0x02);				// Divider M = 2, PLL = 360/(M+1) = 120MHz
	FSMC_LcdWriteData(0x54);				// Validate M and N values
//	Set_Cs;

	FSMC_LcdWriteCmd (0xE0);				// Start PLL command
//	Clr_Cs;
	FSMC_LcdWriteData(0x01);				// enable PLL
//	Set_Cs;

	LCD_Delay(1);						// wait stablize
	FSMC_LcdWriteCmd(0xE0);				// Start PLL command again
//	Clr_Cs;
	FSMC_LcdWriteData(0x03);				// now, use PLL output as system clock
//	Set_Cs;

	FSMC_LcdWriteCmd(0x01);				// Soft reset
	LCD_Delay(10);

	//Set LSHIFT freq, i.e. the DCLK with PLL freq 120MHz set previously
	//Typical DCLK for TY700TFT800480 is 33.3MHz(datasheet), experiment shows 30MHz gives a stable result
	//30MHz = 120MHz*(LCDC_FPR+1)/2^20
	//LCDC_FPR = 262143 (0x3FFFF)
	//Time per line = (DISP_HOR_RESOLUTION+DISP_HOR_PULSE_WIDTH+DISP_HOR_BACK_PORCH+DISP_HOR_FRONT_PORCH)/30 us = 1056/30 = 35.2us
	FSMC_LcdWriteCmd(0xE6);
//	Clr_Cs;
	FSMC_LcdWriteData(0x04);
	FSMC_LcdWriteData(0xff);
	FSMC_LcdWriteData(0xff);

//	Set_Cs;

	//Set panel mode, varies from individual manufacturer
	FSMC_LcdWriteCmd(0xB0);

//	Clr_Cs;
	FSMC_LcdWriteData(0x10);				// set 18-bit for 7" panel TY700TFT800480
	FSMC_LcdWriteData(0x80);				// set TTL mode
	FSMC_LcdWriteData((DISP_HOR_RESOLUTION-1)>>8); //Set panel size
	FSMC_LcdWriteData(DISP_HOR_RESOLUTION-1);
	FSMC_LcdWriteData((DISP_VER_RESOLUTION-1)>>8);
	FSMC_LcdWriteData(DISP_VER_RESOLUTION-1);
	FSMC_LcdWriteData(0x00);				//RGB sequence
//	Set_Cs;

	//Set horizontal period
	FSMC_LcdWriteCmd(0xB4);
	#define HT (DISP_HOR_RESOLUTION+DISP_HOR_PULSE_WIDTH+DISP_HOR_BACK_PORCH+DISP_HOR_FRONT_PORCH)
//	Clr_Cs;
	FSMC_LcdWriteData((HT-1)>>8);
	FSMC_LcdWriteData(HT-1);
	#define HPS (DISP_HOR_PULSE_WIDTH+DISP_HOR_BACK_PORCH)
	FSMC_LcdWriteData((HPS-1)>>8);
	FSMC_LcdWriteData(HPS-1);
	FSMC_LcdWriteData(DISP_HOR_PULSE_WIDTH-1);
	FSMC_LcdWriteData(0x00);
	FSMC_LcdWriteData(0x00);
	FSMC_LcdWriteData(0x00);
//	Set_Cs;

	//Set vertical period
	FSMC_LcdWriteCmd(0xB6);
	#define VT (DISP_VER_PULSE_WIDTH+DISP_VER_BACK_PORCH+DISP_VER_FRONT_PORCH+DISP_VER_RESOLUTION)
//	Clr_Cs;
	FSMC_LcdWriteData((VT-1)>>8);
	FSMC_LcdWriteData(VT-1);
	#define VSP (DISP_VER_PULSE_WIDTH+DISP_VER_BACK_PORCH)
	FSMC_LcdWriteData((VSP-1)>>8);
	FSMC_LcdWriteData(VSP-1);
	FSMC_LcdWriteData(DISP_VER_PULSE_WIDTH-1);
	FSMC_LcdWriteData(0x00);
	FSMC_LcdWriteData(0x00);
//	Set_Cs;

	//Set pixel format, i.e. the bpp
	FSMC_LcdWriteCmd(0x3A);
//	Clr_Cs;
	FSMC_LcdWriteData(0x50); 				// set 16bpp
//	Set_Cs;

	//rotate screen and rgb
	FSMC_LcdWriteCmd(0x36);
//	Clr_Cs;
	FSMC_LcdWriteData(0x0A); 				// set 16bpp
//	Set_Cs;

	//Set pixel data interface
	FSMC_LcdWriteCmd(0xF0);
//	Clr_Cs;
	FSMC_LcdWriteData(0x03);				//16-bit(565 format) data for 16bpp PIC32MX only
	//FSMC_LcdWriteData(0x00);				//8-bit data for 16bpp, PIC24 series
//	Set_Cs;

	FSMC_LcdWriteCmd(0x29);				// Turn on display; show the image on display

/*
	// x51
	FSMC_LcdWriteCmd(0x00E2);	//PLL multiplier, set PLL clock to 120M
	FSMC_LcdWriteData(0x0023);	    //N=0x36 for 6.5M, 0x23 for 10M crystal
	FSMC_LcdWriteData(0x0002);
	FSMC_LcdWriteData(0x0004);
	FSMC_LcdWriteCmd(0x00E0);  // PLL enable
	FSMC_LcdWriteData(0x0001);
	delay_ms(1);
	FSMC_LcdWriteCmd(0x00E0);
	FSMC_LcdWriteData(0x0003);
	delay_ms(5);
	FSMC_LcdWriteCmd(0x0001);  // software reset
	delay_ms(5);
	FSMC_LcdWriteCmd(0x00E6);	//PLL setting for PCLK, depends on resolution
	FSMC_LcdWriteData(0x0003);
	FSMC_LcdWriteData(0x00ff);
	FSMC_LcdWriteData(0x00ff);

	FSMC_LcdWriteCmd(0x00B0);	//LCD SPECIFICATION
	FSMC_LcdWriteData(0x0027);
	FSMC_LcdWriteData(0x0000);
	FSMC_LcdWriteData((HDP>>8)&0X00FF);  //Set HDP
	FSMC_LcdWriteData(HDP&0X00FF);
    FSMC_LcdWriteData((VDP>>8)&0X00FF);  //Set VDP
	FSMC_LcdWriteData(VDP&0X00FF);
    FSMC_LcdWriteData(0x0000);

	FSMC_LcdWriteCmd(0x00B4);	//HSYNC
	FSMC_LcdWriteData((HT>>8)&0X00FF);  //Set HT
	FSMC_LcdWriteData(HT&0X00FF);
	FSMC_LcdWriteData((HPS>>8)&0X00FF);  //Set HPS
	FSMC_LcdWriteData(HPS&0X00FF);
	FSMC_LcdWriteData(HPW);			   //Set HPW
	FSMC_LcdWriteData((LPS>>8)&0X00FF);  //Set HPS
	FSMC_LcdWriteData(LPS&0X00FF);
	FSMC_LcdWriteData(0x0000);

	FSMC_LcdWriteCmd(0x00B6);	//VSYNC
	FSMC_LcdWriteData((VT>>8)&0X00FF);   //Set VT
	FSMC_LcdWriteData(VT&0X00FF);
	FSMC_LcdWriteData((VPS>>8)&0X00FF);  //Set VPS
	FSMC_LcdWriteData(VPS&0X00FF);
	FSMC_LcdWriteData(VPW);			   //Set VPW
	FSMC_LcdWriteData((FPS>>8)&0X00FF);  //Set FPS
	FSMC_LcdWriteData(FPS&0X00FF);

	FSMC_LcdWriteCmd(0x00BA);
	FSMC_LcdWriteData(0x000F);    //GPIO[3:0] out 1

	FSMC_LcdWriteCmd(0x00B8);
	FSMC_LcdWriteData(0x0007);    //GPIO3=input, GPIO[2:0]=output
	FSMC_LcdWriteData(0x0001);    //GPIO0 normal

	FSMC_LcdWriteCmd(0x0036); //rotation
	FSMC_LcdWriteData(0x0000);

	FSMC_LcdWriteCmd(0x00F0); //pixel data interface
	FSMC_LcdWriteData(0x0003);

	delay_ms(5);

	FSMC_LcdWriteCmd(0x0029); //display on

	FSMC_LcdWriteCmd(0x00d0);
	FSMC_LcdWriteData(0x000d);
*/
}


/*********************************************************************
* Function:  LCD_SetArea(start_x,start_y,end_x,end_y)
* PreCondition: SetActivePage(page)
* Input: start_x, end_x	- start column and end column
*		 start_y,end_y 	- start row and end row position (i.e. page address)
* Overview: defines start/end columns and start/end rows for memory access
*			from host to SSD1963
********************************************************************/
void LCD_SetArea(uint16_t start_x, uint16_t start_y, uint16_t end_x, uint16_t end_y)
{
	long offset;

	offset = (uint16_t)_activePage*(GetMaxY()+1);

	start_y = offset + start_y;
	end_y   = offset + end_y;

	FSMC_LcdWriteCmd(CMD_SET_COLUMN);
//	Clr_Cs;
	FSMC_LcdWriteData(start_x>>8);
	FSMC_LcdWriteData(start_x);
	FSMC_LcdWriteData(end_x>>8);
	FSMC_LcdWriteData(end_x);
//	Set_Cs;
	FSMC_LcdWriteCmd(CMD_SET_PAGE);
//	Clr_Cs;
	FSMC_LcdWriteData(start_y>>8);
	FSMC_LcdWriteData(start_y);
	FSMC_LcdWriteData(end_y>>8);
	FSMC_LcdWriteData(end_y);
//	Set_Cs;
}

/*********************************************************************
* Function:  SetScrollArea(SHORT top, SHORT scroll, SHORT bottom)
* Input: top - Top Fixed Area in number of lines from the top
*				of the frame buffer
*		 scroll - Vertical scrolling area in number of lines
*		 bottom - Bottom Fixed Area in number of lines
* Note: Reference: section 9.22 Set Scroll Area, SSD1963 datasheet Rev0.20
********************************************************************/
void LCD_SetScrollArea(uint16_t top, uint16_t scroll, uint16_t bottom)
{
	FSMC_LcdWriteCmd(CMD_SET_SCROLL_AREA);
//	Clr_Cs;
	FSMC_LcdWriteData(top>>8);
	FSMC_LcdWriteData(top);
	FSMC_LcdWriteData(scroll>>8);
	FSMC_LcdWriteData(scroll);
	FSMC_LcdWriteData(bottom>>8);
	FSMC_LcdWriteData(bottom);
//	Set_Cs;
}



/*********************************************************************
* Function:  void  SetScrollStart(SHORT line)
* Overview: First, we need to define the scrolling area by SetScrollArea()
*			before using this function. 
* PreCondition: SetScrollArea(SHORT top, SHORT scroll, SHORT bottom)
* Input: line - Vertical scrolling pointer (in number of lines) as 
*		 the first display line from the Top Fixed Area defined in SetScrollArea()
* Note: Example -
*
*		SHORT line=0;
*		SetScrollArea(0,272,0);
*		for(line=0;line<272;line++) {SetScrollStart(line);LCD_Delay(100);}
*		
*		Code above scrolls the whole page upwards in 100ms interval 
*		with page 2 replacing the first page in scrolling
********************************************************************/
void LCD_SetScrollStart(uint16_t line)
{
	FSMC_LcdWriteCmd(CMD_SET_SCROLL_START);
//	Clr_Cs;
	FSMC_LcdWriteData(line>>8);
	FSMC_LcdWriteData(line);
//	Set_Cs;
}



/*********************************************************************
* Function:  void EnterSleepMode (void)
* Overview: SSD1963 enters sleep mode
* Note: Host must wait 5mS after sending before sending next command
********************************************************************/
void LCD_EnterSleepMode (void)
{
	FSMC_LcdWriteCmd(CMD_ENT_SLEEP);
}



/*********************************************************************
* Function:  void ExitSleepMode (void)
* Overview: SSD1963 exit sleep mode
* Note:   none
********************************************************************/
void LCD_ExitSleepMode (void)
{
	FSMC_LcdWriteCmd(CMD_EXIT_SLEEP);
}



/*********************************************************************
* Function:  void  SetBacklight(BYTE intensity)
* Overview: This function makes use of PWM feature of ssd1963 to adjust
*			the backlight intensity. 
* PreCondition: Backlight circuit with shutdown pin connected to PWM output of ssd1963.
* Input: 	(BYTE) intensity from 
*			0x00 (total backlight shutdown, PWM pin pull-down to VSS)
*			0xff (99% pull-up, 255/256 pull-up to VDD)
* Note: The base frequency of PWM set to around 300Hz with PLL set to 120MHz.
*		This parameter is hardware dependent
********************************************************************/
void LCD_SetBacklight(uint8_t intensity)
{
	FSMC_LcdWriteCmd(0xBE);			// Set PWM configuration for backlight control
//	Clr_Cs;
	FSMC_LcdWriteData(0x0E);			// PWMF[7:0] = 2, PWM base freq = PLL/(256*(1+5))/256 =
								// 300Hz for a PLL freq = 120MHz
	FSMC_LcdWriteData(intensity);		// Set duty cycle, from 0x00 (total pull-down) to 0xFF
								// (99% pull-up , 255/256)
	FSMC_LcdWriteData(0x01);			// PWM enabled and controlled by host (mcu)
	FSMC_LcdWriteData(0x00);
	FSMC_LcdWriteData(0x00);
	FSMC_LcdWriteData(0x00);

//	Set_Cs;
}



/*********************************************************************
* Function:  void  SetTearingCfg(uint8_t state, uint8_t mode)
* Overview: This function enable/disable tearing effect
* Input: 	BOOL state -	1 to enable
*				0 to disable
*		BOOL mode -	0:  the tearing effect output line consists
*					of V-blanking information only
*				1:  the tearing effect output line consists
*					of both V-blanking and H-blanking info.
********************************************************************/
void LCD_SetTearingCfg(uint8_t state, uint8_t mode)
{
	if(state == 1)
	{
		FSMC_LcdWriteCmd(0x35);
//		Clr_Cs;
		FSMC_LcdWriteData(mode&0x01);
//		Set_Cs;
	}
	else
	{
		FSMC_LcdWriteCmd(0x34);
	}
}





/*********************************************************************
* Function: void PutPixel(SHORT x, SHORT y)
* Input: x,y - pixel coordinates
********************************************************************/
void LCD_PutPixel(uint16_t x, uint16_t y, uint16_t color)
{
	if(_clipRgn){
		if(x<_clipLeft)
			return;
		if(x>_clipRight)
			return;
		if(y<_clipTop)
			return;
		if(y>_clipBottom)
			return;
	}
  	
	LCD_SetArea(x,y,GetMaxX(),GetMaxY());
	FSMC_LcdWriteCmd(CMD_WR_MEMSTART);
//	Clr_Cs;
	FSMC_LcdWriteData(color);
//	Set_Cs;
}



/*********************************************************************
* Function: void ClearDevice(void)
* Overview:     
********************************************************************/
void LCD_Clear(uint16_t color)
{
	u32 xcounter, ycounter;

	LCD_SetArea(0,0,GetMaxX(),GetMaxY());
	FSMC_LcdWriteCmd(CMD_WR_MEMSTART);

//	Clr_Cs;
	for(ycounter=0;ycounter<GetMaxY()+1;ycounter++)

		for(xcounter=0;xcounter<GetMaxX()+1;xcounter++)
			FSMC_LcdWriteData(color);
//	Set_Cs;
}

void LCD_Delay(vu32 nCount)
{
  nCount = nCount * 1000;
  for(; nCount != 0; nCount--);
}
/*
void LCD_DelayUs(vu32 nCount)
{
  for(; nCount != 0; nCount--);
}
*/
