//#include "stm32f4xx.h"
#include "SSD1963_Configuration.h"


#ifndef _SSD1963_H
#define _SSD1963_H

//include the command table for SSD1963
#include "SSD1963_CMD.h"


/*********************************************************************
* Overview: Clipping region control codes to be used with SetClip(...)
*           function. 
*********************************************************************/
#define CLIP_DISABLE       0 	// Disables clipping.
#define CLIP_ENABLE        1	// Enables clipping.


/*********************************************************************
* Macros: RGB565CONVERT(red, green, blue)
*  R,G,B  16  5:6:5
********************************************************************/
#define RGB565CONVERT(red, green, blue) (uint16_t) (((red >> 3) << 11) | ((green >> 2) << 5) | (blue >> 3))
    

/*********************************************************************
* Overview:  .
*********************************************************************/
#define BLACK               RGB565CONVERT(0,    0,      0)
#define BRIGHTBLUE          RGB565CONVERT(0,    0,      255)
#define BRIGHTGREEN         RGB565CONVERT(0,    255,    0)
#define BRIGHTCYAN          RGB565CONVERT(0,    255,    255)
#define BRIGHTRED           RGB565CONVERT(255,  0,      0)
#define BRIGHTMAGENTA       RGB565CONVERT(255,  0,      255)
#define BRIGHTYELLOW        RGB565CONVERT(255,  255,    0)
#define BLUE                RGB565CONVERT(0,    0,      128)
#define GREEN               RGB565CONVERT(0,    128,    0)
#define CYAN                RGB565CONVERT(0,    128,    128)
#define RED                 RGB565CONVERT(128,  0,      0)
#define MAGENTA             RGB565CONVERT(128,  0,      128)
#define BROWN               RGB565CONVERT(255,  128,    0)
#define LIGHTGRAY           RGB565CONVERT(128,  128,    128)
#define DARKGRAY            RGB565CONVERT(64,   64,     64)
#define LIGHTBLUE           RGB565CONVERT(128,  128,    255)
#define LIGHTGREEN          RGB565CONVERT(128,  255,    128)
#define LIGHTCYAN           RGB565CONVERT(128,  255,    255)
#define LIGHTRED            RGB565CONVERT(255,  128,    128)
#define LIGHTMAGENTA        RGB565CONVERT(255,  128,    255)
#define YELLOW              RGB565CONVERT(255,  255,    128)
#define WHITE               RGB565CONVERT(255,  255,    255)
                            
#define GRAY0       	    RGB565CONVERT(224,  224,    224)
#define GRAY1         	    RGB565CONVERT(192,  192,    192)   
#define GRAY2               RGB565CONVERT(160,  160,    160)   
#define GRAY3               RGB565CONVERT(128,  128,    128)
#define GRAY4               RGB565CONVERT(96,   96,     96)
#define GRAY5               RGB565CONVERT(64,   64,     64)
#define GRAY6	            RGB565CONVERT(32,   32,     32)

/*
#define Set_Cs  GPIOC->ODR |= 0x0001; //pin0
#define Clr_Cs  GPIOC->ODR &= ~0x0001;

#define Set_Rs  GPIOA->ODR |= 0x2000; //pin13
#define Clr_Rs  GPIOA->ODR &= ~0x2000;

#define Set_nWr GPIOA->ODR |= 0x4000; //pin14
#define Clr_nWr GPIOA->ODR &= ~0x4000;

#define Set_Rst GPIOA->ODR |= 0x8000; //pin15
#define Clr_Rst GPIOA->ODR &= ~0x8000;
*/

/*********************************************************************
* Overview: Clipping region control and border settings.
*********************************************************************/
// Clipping region enable control
extern uint16_t _clipRgn;

// Left clipping region border
extern uint16_t _clipLeft;
// Top clipping region border
extern uint16_t _clipTop;
// Right clipping region border
extern uint16_t _clipRight;
// Bottom clipping region border
extern uint16_t _clipBottom;


// Active and Visual Pages 
extern uint8_t 	_activePage;
extern uint8_t	_visualPage;


/*********************************************************************
* Macros:  LCD_WriteCommand(cmd)                                      LCD_DelayUs(5);
********************************************************************/
//#define LCD_WriteCommand(cmd) {Clr_Rs; GPIOB->ODR=cmd; Clr_Cs; Clr_nWr; Set_nWr; Set_Cs;};

	
/*********************************************************************
* Function:  void  LCD_WriteData(WORD data)
********************************************************************/
//#define LCD_WriteData(data) {Set_Rs; GPIOB->ODR=data; Clr_nWr; Set_nWr;};

void LCD_Clear(uint16_t color);

/*********************************************************************
* Function:  void ResetDevice()
* Overview: Initializes LCD module.
********************************************************************/
void LCD_Initialize(void);




/*********************************************************************
* Macros:  GetMaxX()
* Overview:      
********************************************************************/
#if (DISP_ORIENTATION == 90) || (DISP_ORIENTATION == 270)
#define	GetMaxX() 			(DISP_VER_RESOLUTION-1)
	

#elif (DISP_ORIENTATION == 0) || (DISP_ORIENTATION == 180)
#define	GetMaxX() 			(DISP_HOR_RESOLUTION-1)

#endif




/*********************************************************************
* Macros:  GetMaxY()
* Overview:      Y
********************************************************************/
#if (DISP_ORIENTATION == 90) || (DISP_ORIENTATION == 270)
#define	GetMaxY() 			(DISP_HOR_RESOLUTION-1)

#elif (DISP_ORIENTATION == 0) || (DISP_ORIENTATION == 180)
#define	GetMaxY() 			(DISP_VER_RESOLUTION-1)

#endif


//            
// The maximum memory space of SSD1928 is 262kbyte
#ifndef PAGE_MEM_SIZE
// Memory allocation for a single page 
#define PAGE_MEM_SIZE 	(GetMaxX()+1)*(GetMaxY()+1)*COLOR_DEPTH/8

#if PAGE_MEM_SIZE > 1215000ul	//this is the memory boundary of ssd1963
#error MEMORY ALLOCATION FAILED. CHECK SSD1963.h
#endif

#endif





/*********************************************************************
* Macros:  SetActivePage(page)
* Overview:    .
********************************************************************/
#define LCD_SetActivePage(page)	_activePage = page




/*********************************************************************
* Macros: SetVisualPage(page)
* Overview:     .
********************************************************************/
//debug
#define LCD_SetVisualPage(page) {_visualPage = page; \
LCD_SetScrollArea(0,LCD_GetMaxY()+1,0); \
LCD_SetScrollStart((SHORT)_visualPage*(LCD_GetMaxY()+1));}
//debug





/*********************************************************************
* Function: void PutPixel(SHORT x, SHORT y, color)
* Overview:     X, Y  .
********************************************************************/
void LCD_PutPixel(uint16_t x, uint16_t y, uint16_t color);




/*********************************************************************
* Macros: SetClipRgn(left, top, right, bottom)
* Overview: Sets clipping region.   
* Input: left - Defines the left clipping region border.
*		 top - Defines the top clipping region border.
*		 right - Defines the right clipping region border.
*	     bottom - Defines the bottom clipping region border.
********************************************************************/
#define LCD_SetClipRgn(left,top,right,bottom) _clipLeft=left; _clipTop=top; _clipRight=right; _clipBottom=bottom;





/*********************************************************************
* Macros: GetClipLeft()
* Overview: Returns left clipping border.
********************************************************************/
#define LCD_GetClipLeft() _clipLeft




/*********************************************************************
* Macros: GetClipRight()
* Overview: Returns right clipping border.
********************************************************************/
#define LCD_GetClipRight() _clipRight



/*********************************************************************
* Macros: GetClipTop()
* Overview: Returns top clipping border.
********************************************************************/
#define LCD_GetClipTop() _clipTop



/*********************************************************************
* Macros: GetClipBottom()
* Overview: Returns bottom clipping border.
********************************************************************/
#define LCD_GetClipBottom() _clipBottom



/*********************************************************************
* Macros: SetClip(control)
* Overview: Enables/disables clipping.
* Input: control - Enables or disables the clipping.
*			- 0: Disable clipping
*			- 1: Enable clipping
********************************************************************/
#define LCD_SetClip(control) _clipRgn=control;



/*********************************************************************
* Macros: IsDeviceBusy()
* Overview:         
*           ( ).
********************************************************************/
#define LCD_IsDeviceBusy()  0



/*********************************************************************
* Macros: SetPalette(colorNum, color)
* Overview:  Sets palette register.
* Input: colorNum - Register number.
*        color - Color.
********************************************************************/
#define LCD_SetPalette(colorNum, color)



/*********************************************************************
* Function:  SetScrollArea(SHORT top, SHORT scroll, SHORT bottom)
* Input: top - Top Fixed Area in number of lines from the top
*				of the frame buffer
*		 scroll - Vertical scrolling area in number of lines
*		 bottom - Bottom Fixed Area in number of lines
*
* Note: Reference: section 9.22 Set Scroll Area, SSD1963 datasheet Rev0.20
*
********************************************************************/
void LCD_SetScrollArea(uint16_t top, uint16_t scroll, uint16_t bottom);



/*********************************************************************
* Function:  void  SetScrollStart(SHORT line)
* Overview: First, we need to define the scrolling area by SetScrollArea()
*			before using this function. 
* PreCondition: SetScrollArea(SHORT top, SHORT scroll, SHORT bottom)
* Input: line - Vertical scrolling pointer (in number of lines) as 
*		 the first display line from the Top Fixed Area defined in SetScrollArea()
* Note: Example -
*
*		SHORT line=0;
*		SetScrollArea(0,272,272);
*		for(line=0;line<272;line++) {SetScrollStart(line);DelayMs(100);}
*		
*		Code above scrolls the whole page upwards in 100ms interval 
*		with page 2 replacing the first page in scrolling
********************************************************************/
void LCD_SetScrollStart(uint16_t line);




/*********************************************************************
* Function:  void EnterSleepMode (void)
* Note: Host must wait 5mS after sending before sending next command
********************************************************************/
void LCD_EnterSleepMode (void);



/*********************************************************************
* Function:  void ExitSleepMode (void)
* Note:   cannot be called sooner than 15
********************************************************************/
void LCD_ExitSleepMode (void);



/*********************************************************************
* Function:  void  SetBacklight(BYTE intensity)
* Overview: This function makes use of PWM feature of ssd1963 to adjust
*			the backlight intensity. 
*
* PreCondition: Backlight circuit with shutdown pin connected to PWM output of ssd1963.
* Input: 	(BYTE) intensity from 
*			0x00 (total backlight shutdown, PWM pin pull-down to VSS)
*			0xff (99% pull-up, 255/256 pull-up to VDD)
* Note: The base frequency of PWM set to around 300Hz with PLL set to 120MHz.
*		This parameter is hardware dependent
********************************************************************/
void LCD_SetBacklight(uint8_t intensity);



/*********************************************************************
* Function:  void  SetTearingCfg(uint8_t state, uint8_t mode)
* Overview: This function enable/disable tearing effect
* Input: 	BOOL state -	1 to enable
*				0 to disable
*		BOOL mode -	0:the tearing effect output line consists
*				  of V-blanking information only
*				1:the tearing effect output line consists
*				  of both V-blanking and H-blanking info.
********************************************************************/
void LCD_SetTearingCfg(uint8_t state, uint8_t mode);



/************************************************************************
* Macro: Lo                                                             *
* Preconditions: None                                                   *
* Overview: This macro extracts a low byte from a 2 byte word.          *
************************************************************************/
#define Lo(X)   (uint8_t)(X&0x00ff)



/************************************************************************
* Macro: Hi                                                             *
* Preconditions: None                                                   *
* Overview: This macro extracts a high byte from a 2 byte word.         *
************************************************************************/
#define Hi(X)   (uint8_t)((X>>8)&0x00ff)


#endif // _SSD1963_H
