// FSMC_LowLevel.c -   FSMC  LCD

#include <stdint.h>

#include "stm32f4xx.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_fsmc.h"
#include "stm32f4xx_rcc.h"

#include "System.h"

#include "FSMC_LowLevel.h"

//  SSD1963
//  RS = 1 - 
//  RS = 0 - 

// STM324x7I_EVAL_LCD_Private_TypesDefinitions
typedef struct
{
  volatile uint16_t LCD_REG;
  volatile uint16_t LCD_RAM;
} LCD_TypeDef;

// Note: LCD /CS is NE1 - Bank 1 of NOR/SRAM Bank 1~4
#define LCD_BASE           ((uint32_t)(0x60000000 | 0x0001fffE))
#define LCD                ((LCD_TypeDef *) LCD_BASE)

//   FSMC
static void FSMC_LinesConfig ( void );
//  FSMC
static void FSMC_Config ( void );

//  FSMC
void FSMC_Init ( void )
{
	//   FSMC
	FSMC_LinesConfig ( );

	//  FSMC
	FSMC_Config ( );
} // FSMC_Init

//    LCD
void FSMC_LcdWriteReg ( uint8_t LCD_Reg, uint16_t LCD_RegValue )
{
	// Write 16-bit Index, then Write Reg
	LCD->LCD_REG = LCD_Reg;
	// Write 16-bit Reg
	LCD->LCD_RAM = LCD_RegValue;
} // FSMC_LcdWriteReg

//   LCD
uint16_t FSMC_LcdReadReg ( uint8_t LCD_Reg )
{
	// Write 16-bit Index (then Read Reg)
	LCD->LCD_REG = LCD_Reg;
	// Read 16-bit Reg
	return (LCD->LCD_RAM);
} // FSMC_LcdReadReg

//    LCD
void FSMC_LcdWriteCmd ( uint16_t val )
{
	// Write 16-bit Index (then Read Reg)
	LCD->LCD_REG = val;
} // FSMC_LcdWriteCmd

//    LCD
void FSMC_LcdWriteData ( uint16_t val )
{
	// Write 16-bit Reg
	LCD->LCD_RAM = val;
} // FSMC_LcdWriteData

/////////////////////////////////////////////////////////////
//  

//   FSMC
static void FSMC_LinesConfig ( void )
{
	GPIO_InitTypeDef GPIO_InitStructure;

	// Enable GPIOD, GPIOE clocks
	RCC_AHB1PeriphClockCmd ( RCC_AHB1Periph_GPIOD | RCC_AHB1Periph_GPIOE, ENABLE );

	// -- GPIO Configuration ------------------------------------------------------
	// LCD DB15-DB0       PD10..PD8, PE15..PE7, PD1, PD0, PD15, PD14
	// LCD nCS  (7)       NCE2 (PD7)
	// LCD nRD  (10)      NOE  (PD4)
	// LCD nWR  (9)       NWE  (PD5)
	// LCD RS   (8)       CLE  (PD11)

	// SRAM Data lines,  NOE and NWE configuration
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_8 | GPIO_Pin_9 |
			GPIO_Pin_10 | GPIO_Pin_14 | GPIO_Pin_15 | GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_7 | GPIO_Pin_11;

	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;
	GPIO_Init ( GPIOD, &GPIO_InitStructure );

	GPIO_PinAFConfig ( GPIOD, GPIO_PinSource0, GPIO_AF_FSMC );
	GPIO_PinAFConfig ( GPIOD, GPIO_PinSource1, GPIO_AF_FSMC );
	GPIO_PinAFConfig ( GPIOD, GPIO_PinSource4, GPIO_AF_FSMC );
	GPIO_PinAFConfig ( GPIOD, GPIO_PinSource5, GPIO_AF_FSMC );
	GPIO_PinAFConfig ( GPIOD, GPIO_PinSource7, GPIO_AF_FSMC );
	GPIO_PinAFConfig ( GPIOD, GPIO_PinSource8, GPIO_AF_FSMC );
	GPIO_PinAFConfig ( GPIOD, GPIO_PinSource9, GPIO_AF_FSMC );
	GPIO_PinAFConfig ( GPIOD, GPIO_PinSource10, GPIO_AF_FSMC );
	GPIO_PinAFConfig ( GPIOD, GPIO_PinSource11, GPIO_AF_FSMC );
	GPIO_PinAFConfig ( GPIOD, GPIO_PinSource14, GPIO_AF_FSMC );
	GPIO_PinAFConfig ( GPIOD, GPIO_PinSource15, GPIO_AF_FSMC );

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10 |
			GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;
	GPIO_Init ( GPIOE, &GPIO_InitStructure );

	GPIO_PinAFConfig ( GPIOE, GPIO_PinSource7, GPIO_AF_FSMC );
	GPIO_PinAFConfig ( GPIOE, GPIO_PinSource8, GPIO_AF_FSMC );
	GPIO_PinAFConfig ( GPIOE, GPIO_PinSource9, GPIO_AF_FSMC );
	GPIO_PinAFConfig ( GPIOE, GPIO_PinSource10, GPIO_AF_FSMC );
	GPIO_PinAFConfig ( GPIOE, GPIO_PinSource11, GPIO_AF_FSMC );
	GPIO_PinAFConfig ( GPIOE, GPIO_PinSource12, GPIO_AF_FSMC );
	GPIO_PinAFConfig ( GPIOE, GPIO_PinSource13, GPIO_AF_FSMC );
	GPIO_PinAFConfig ( GPIOE, GPIO_PinSource14, GPIO_AF_FSMC );
	GPIO_PinAFConfig ( GPIOE, GPIO_PinSource15, GPIO_AF_FSMC );
} // FSMC_LinesConfig

//  FSMC
static void FSMC_Config ( void )
{
	FSMC_NORSRAMInitTypeDef  FSMC_NORSRAMInitStructure;
	FSMC_NORSRAMTimingInitTypeDef  p;

	// Enable FSMC clock
	RCC_AHB3PeriphClockCmd ( RCC_AHB3Periph_FSMC, ENABLE );

	/*-- FSMC Configuration ------------------------------------------------------*/
	/*----------------------- SRAM Bank 3 ----------------------------------------*/
	/* FSMC_Bank1_NORSRAM4 configuration */
	p.FSMC_AddressSetupTime = 0x01;
	p.FSMC_AddressHoldTime = 0x01;
	p.FSMC_DataSetupTime = 0x04;
	p.FSMC_BusTurnAroundDuration = 0x00;
	p.FSMC_CLKDivision = 0x00;
	p.FSMC_DataLatency = 0x00;
	p.FSMC_AccessMode = FSMC_AccessMode_A;

	FSMC_NORSRAMInitStructure.FSMC_Bank = FSMC_Bank1_NORSRAM1;
	FSMC_NORSRAMInitStructure.FSMC_DataAddressMux = FSMC_DataAddressMux_Disable;
	FSMC_NORSRAMInitStructure.FSMC_MemoryType = FSMC_MemoryType_SRAM;
	FSMC_NORSRAMInitStructure.FSMC_MemoryDataWidth = FSMC_MemoryDataWidth_16b;
	FSMC_NORSRAMInitStructure.FSMC_BurstAccessMode = FSMC_BurstAccessMode_Disable;
	FSMC_NORSRAMInitStructure.FSMC_WaitSignalPolarity = FSMC_WaitSignalPolarity_Low;
	FSMC_NORSRAMInitStructure.FSMC_WrapMode = FSMC_WrapMode_Disable;
	FSMC_NORSRAMInitStructure.FSMC_WaitSignalActive = FSMC_WaitSignalActive_BeforeWaitState;
	FSMC_NORSRAMInitStructure.FSMC_WriteOperation = FSMC_WriteOperation_Enable;
	FSMC_NORSRAMInitStructure.FSMC_WaitSignal = FSMC_WaitSignal_Disable;
	FSMC_NORSRAMInitStructure.FSMC_AsynchronousWait = FSMC_AsynchronousWait_Disable; //
	FSMC_NORSRAMInitStructure.FSMC_ExtendedMode = FSMC_ExtendedMode_Disable;
	FSMC_NORSRAMInitStructure.FSMC_WriteBurst = FSMC_WriteBurst_Disable;
	FSMC_NORSRAMInitStructure.FSMC_ReadWriteTimingStruct = &p;
	FSMC_NORSRAMInitStructure.FSMC_WriteTimingStruct = &p;
	FSMC_NORSRAMInit ( &FSMC_NORSRAMInitStructure );

	// Enable FSMC Bank1_SRAM Bank
	FSMC_NORSRAMCmd ( FSMC_Bank1_NORSRAM1, ENABLE );
//	FSMC_NORSRAMInit ( &FSMC_NORSRAMInitStructure );
} // FSMC_Config
