// SSD1963_Drv.c -   SSD1963 (FSMC)

#include <string.h>
#include <stdbool.h>
#include <stdint.h>
#include <stdlib.h>		// abs

#include "stm32f4xx_gpio.h"
#include "stm32f4xx_rcc.h"

#include "SSD1963_Drv.h"

//#include "delay_ms.h"
#include "System.h"
#include "fontLucidaConsole12pt.h"

#include "FSMC_LowLevel.h"

//   (  )
#define DISP_WIDTH  800
#define DISP_HEIGHT 480


// *********************************************************************
// * Overview: Horizontal synchronization timing in pixels
// *                  (from the glass datasheet).
// *********************************************************************
#define DISP_HOR_PULSE_WIDTH		1
#define DISP_HOR_BACK_PORCH			210
#define DISP_HOR_FRONT_PORCH		45

// *********************************************************************
// * Overview: Vertical synchronization timing in lines
// *                  (from the glass datasheet).
// ********************************************************************
#define DISP_VER_PULSE_WIDTH		1
#define DISP_VER_BACK_PORCH			34
#define DISP_VER_FRONT_PORCH		10

#define HT (DISP_WIDTH+DISP_HOR_PULSE_WIDTH+DISP_HOR_BACK_PORCH+DISP_HOR_FRONT_PORCH)
#define HPS (DISP_HOR_PULSE_WIDTH+DISP_HOR_BACK_PORCH)

#define VT (DISP_VER_PULSE_WIDTH+DISP_VER_BACK_PORCH+DISP_VER_FRONT_PORCH+DISP_HEIGHT)
#define VSP (DISP_VER_PULSE_WIDTH+DISP_VER_BACK_PORCH)

/*
// x51
static const uint16_t HDP = DISP_WIDTH - 1;
static const uint16_t HT = 928;
static const uint16_t HPS = 46;
static const uint16_t LPS = 15;
static const uint16_t HPW = 48;

static const uint16_t VDP = DISP_HEIGHT - 1;
static const uint16_t VT = 525;
static const uint16_t VPS =16;
static const uint16_t FPS = 8;
static const uint16_t VPW = 16;
*/

/////////////////////////////////////////////////////////////////////////////////////////////
//     

//     (,    )
static void PutPixelInternal ( uint16_t x, uint16_t y, uint16_t color );

//
/////////////////////////////////////////////////////////////////////////////////////////////

//  
static TOrientation orient = Orientation_Portrait;

//   (fontLucidaConsole12pt.c)
extern const FONT_INFO lucidaConsole12ptFontInfo;
//const FONT_INFO *font = &lucidaConsole12ptFontInfo;
const FONT_INFO *font = NULL;

//  
void SSD1963_Init ( void )
{
	GPIO_InitTypeDef GPIO_InitStructure;

	//  Reset - PD2
	//        nCS - PD7
	//        nWR - PD5

	// Enable GPIOD clocks
	RCC_AHB1PeriphClockCmd ( RCC_AHB1Periph_GPIOD, ENABLE );

	// PD2, PD5, PD7
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2 | GPIO_Pin_5 | GPIO_Pin_7;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
	GPIO_Init ( GPIOD, &GPIO_InitStructure );

	// PD2 := PD5 := 1, PD7 := 1 - nRES := nCS := nWR := 1
	GPIO_SetBits ( GPIOD, GPIO_Pin_2 | GPIO_Pin_5 | GPIO_Pin_7 );

	// 
	delay_ms ( 5 );
	// nRES := 0
	GPIO_ResetBits ( GPIOD, GPIO_Pin_2 );
	delay_ms ( 15 );
	// nRES := 1
	GPIO_SetBits ( GPIOD, GPIO_Pin_2 );
	delay_ms ( 15 );

	// 5ms  SSD1963   
//	delay_ms ( 5 );

	//  FSMC
	FSMC_Init ( );
/*
	FSMC_LcdWriteCmd(0x0001);
	delay_ms(500);
	FSMC_LcdWriteCmd(0x0011);
	delay_ms(500);
	FSMC_LcdWriteCmd(0x0029);
	delay_ms(500);

	  FSMC_LcdWriteCmd(0x00E2);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x0021);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x0002);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x0004);
	  delay_ms(500);


	  FSMC_LcdWriteCmd(0x00E0);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x0001);
	  delay_ms(500);
	  FSMC_LcdWriteCmd(0x00E0);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x0003);
	  delay_ms(500);

	  FSMC_LcdWriteCmd(0x0001);
	  delay_ms(500);

	  FSMC_LcdWriteCmd(0x00E6);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x0002);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x0048);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x009E);
	  delay_ms(500);


	  FSMC_LcdWriteCmd(0x00B0);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x0028);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x0000);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x0001);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x003F);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x0000);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x00EF);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x0000);
	  delay_ms(500);

	  FSMC_LcdWriteCmd(0x00B4);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x0001);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x00B8);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x0000);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x0046);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x000F);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x0000);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x0000);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x0000);

	  FSMC_LcdWriteCmd(0x00B6);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x0001);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x0048);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x0000);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x000B);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x0007);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x0000);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x0000);
	  delay_ms(500);

	  FSMC_LcdWriteCmd(0x002A);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x0000);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x0000);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x0001);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x003F);
	  delay_ms(500);

	  FSMC_LcdWriteCmd(0x002B);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x0000);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x0000);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x0000);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x00EF);
	  delay_ms(500);

	  FSMC_LcdWriteCmd(0x00F0);
	  delay_ms(500);
	  FSMC_LcdWriteData(0x0003);
//	  delay_ms(500);

	  FSMC_LcdWriteCmd(0x003A);
//	  delay_ms(500);
	  FSMC_LcdWriteData(0x0003);
//	  delay_ms(500);

	  FSMC_LcdWriteCmd(0x0013);
//	  delay_ms(500);
	  FSMC_LcdWriteCmd(0x0029);
//	  delay_ms(500);
*/


	FSMC_LcdWriteCmd ( 0xE2 );			// Set PLL with OSC = 10MHz (hardware)
	FSMC_LcdWriteData ( 0x23 );
	FSMC_LcdWriteData ( 0x02 );			// Divider M = 2, PLL = 360/(M+1) = 120MHz
	FSMC_LcdWriteData ( 0x54 );			// Validate M and N values

    delay_ms ( 100 );
    FSMC_LcdWriteCmd ( 0x01 );     //Software Reset
    delay_ms (100);
    FSMC_LcdWriteCmd ( 0x01 );
    FSMC_LcdWriteCmd ( 0x01 );
    delay_ms ( 10 );

	FSMC_LcdWriteCmd ( 0xE0 );			// Start PLL command
	FSMC_LcdWriteData ( 0x01 );			// enable PLL

	delay_ms ( 1 );

	FSMC_LcdWriteCmd ( 0xE0 );			// Start PLL command
	FSMC_LcdWriteData ( 0x03 );			// now, use PLL output as system clock
	FSMC_LcdWriteCmd ( 0x01 );			// Soft reset

	//Set LSHIFT freq, i.e. the DCLK with PLL freq 120MHz set previously
	//Typical DCLK for TY700TFT800480 is 33.3MHz(datasheet), experiment shows 30MHz gives a stable result
	//30MHz = 120MHz*(LCDC_FPR+1)/2^20
	//LCDC_FPR = 262143 (0x3FFFF)
	//Time per line = (DISP_HOR_RESOLUTION+DISP_HOR_PULSE_WIDTH+DISP_HOR_BACK_PORCH+DISP_HOR_FRONT_PORCH)/30 us = 1056/30 = 35.2us
	FSMC_LcdWriteCmd ( 0xE6 );
	FSMC_LcdWriteData ( 0x04 );
	FSMC_LcdWriteData ( 0xff );
	FSMC_LcdWriteData ( 0xff );

	//Set panel mode, varies from individual manufacturer
	FSMC_LcdWriteCmd ( 0xB0 );
	FSMC_LcdWriteData ( 0x10 );				// set 18-bit for 7" panel TY700TFT800480
	FSMC_LcdWriteData ( 0x80 );				// set TTL mode
	FSMC_LcdWriteData ( (DISP_WIDTH-1)>>8); //Set panel size
	FSMC_LcdWriteData ( DISP_WIDTH-1);
	FSMC_LcdWriteData ( (DISP_HEIGHT-1)>>8);
	FSMC_LcdWriteData ( DISP_HEIGHT - 1 );
	FSMC_LcdWriteData ( 0x00 );				//RGB sequence

	//Set horizontal period
	FSMC_LcdWriteData ( 0xB4 );
	FSMC_LcdWriteData ( (HT-1) >> 8 );
	FSMC_LcdWriteData ( HT-1 );
	FSMC_LcdWriteData ( (HPS-1) >> 8 );
	FSMC_LcdWriteData ( HPS-1 );
	FSMC_LcdWriteData ( DISP_HOR_PULSE_WIDTH-1 );
	FSMC_LcdWriteData ( 0x00 );
	FSMC_LcdWriteData ( 0x00 );
	FSMC_LcdWriteData ( 0x00 );

	//Set vertical period
	FSMC_LcdWriteCmd ( 0xB6 );
	FSMC_LcdWriteData ( (VT-1)>>8 );
	FSMC_LcdWriteData ( VT-1 );
	FSMC_LcdWriteData ( (VSP-1)>>8 );
	FSMC_LcdWriteData ( VSP-1 );
	FSMC_LcdWriteData ( DISP_VER_PULSE_WIDTH-1 );
	FSMC_LcdWriteData ( 0x00 );
	FSMC_LcdWriteData ( 0x00 );

	//Set pixel format, i.e. the bpp
	FSMC_LcdWriteCmd ( 0x3A );
	FSMC_LcdWriteData ( 0x50 ); 				// set 16bpp

	// rotate screen and rgb
	FSMC_LcdWriteCmd ( 0x36 );
	FSMC_LcdWriteData ( 0x0A ); 				// set 16bpp

	//Set pixel data interface
	FSMC_LcdWriteCmd ( 0xF0 );
	FSMC_LcdWriteData ( 0x03 );				//16-bit(565 format) data for 16bpp PIC32MX only
	//LCD_WriteData(0x00);					//8-bit data for 16bpp, PIC24 series
	FSMC_LcdWriteCmd ( 0x29 );				// Turn on display; show the image on display

/*
	// From x51
	FSMC_LcdWriteCmd ( 0x00E2 );				// PLL multiplier, set PLL clock to 120M
	FSMC_LcdWriteData ( 0x0023 );				// N=0x36 for 6.5M, 0x23 for 10M crystal
	FSMC_LcdWriteData ( 0x0002 );
	FSMC_LcdWriteData ( 0x0004 );
	FSMC_LcdWriteCmd ( 0x00E0 );				// PLL enable
	FSMC_LcdWriteData ( 0x0001 );
	delay_ms_ms(1);
	FSMC_LcdWriteCmd ( 0x00E0 );
	FSMC_LcdWriteData ( 0x0003 );
	delay_ms_ms(5);
	FSMC_LcdWriteCmd ( 0x0001 );				// software reset
	delay_ms_ms(5);
	FSMC_LcdWriteCmd ( 0x00E6 );				// PLL setting for PCLK, depends on resolution
	FSMC_LcdWriteData ( 0x0003 );
	FSMC_LcdWriteData ( 0x00ff );
	FSMC_LcdWriteData ( 0x00ff );

	FSMC_LcdWriteCmd ( 0x00B0 );				// LCD SPECIFICATION
	FSMC_LcdWriteData ( 0x0027 );
	FSMC_LcdWriteData ( 0x0000 );
	FSMC_LcdWriteData ( (HDP>>8) & 0x00FF );	// Set HDP
	FSMC_LcdWriteData ( HDP&0x00FF );
    FSMC_LcdWriteData ( (VDP>>8) & 0X00FF );	// Set VDP
	FSMC_LcdWriteData ( VDP&0x00FF );
    FSMC_LcdWriteData ( 0x0000 );

	FSMC_LcdWriteCmd ( 0x00B4);					// HSYNC
	FSMC_LcdWriteData ( (HT>>8) & 0x00FF );		// Set HT
	FSMC_LcdWriteData ( HT & 0x00FF );
	FSMC_LcdWriteData ( (HPS>>8) & 0x00FF );	// Set HPS
	FSMC_LcdWriteData ( HPS&0x00FF );
	FSMC_LcdWriteData ( HPW );					// Set HPW
	FSMC_LcdWriteData ( (LPS>>8) & 0x00FF );	// Set HPS
	FSMC_LcdWriteData ( LPS&0x00FF );
	FSMC_LcdWriteData ( 0x0000 );

	FSMC_LcdWriteCmd ( 0x00B6 );				// VSYNC
	FSMC_LcdWriteData ( (VT>>8) & 0x00FF );		// Set VT
	FSMC_LcdWriteData ( VT & 0x00FF );
	FSMC_LcdWriteData ( (VPS>>8) & 0x00FF );	// Set VPS
	FSMC_LcdWriteData ( VPS & 0x00FF);
	FSMC_LcdWriteData ( VPW );					// Set VPW
	FSMC_LcdWriteData ( (FPS>>8) & 0x00FF );	// Set FPS
	FSMC_LcdWriteData ( FPS & 0x00FF );

	FSMC_LcdWriteCmd ( 0x00BA );
	FSMC_LcdWriteData ( 0x000F );		// GPIO[3:0] out 1

	FSMC_LcdWriteCmd ( 0x00B8 );
	FSMC_LcdWriteData ( 0x0007 );		// GPIO3=input, GPIO[2:0]=output
	FSMC_LcdWriteData ( 0x0001 );		// GPIO0 normal

	FSMC_LcdWriteCmd ( 0x0036 );		// rotation
	FSMC_LcdWriteData ( 0x0000 );

	FSMC_LcdWriteCmd ( 0x00F0 ); 		// pixel data interface
	FSMC_LcdWriteData ( 0x0003 );

	delay_ms_ms(5);

	FSMC_LcdWriteCmd (0x0029); //display on

	FSMC_LcdWriteCmd (0x00d0);
	FSMC_LcdWriteData (0x000d);
*/

/*
	//   
	SSD1963_FillScr ( 0xFFFF );
	// 
	SSD1963_SetOrient ( Orientation_Portrait );

	// 
	SSD1963_SetFont ( LucidaConsole12pt );
*/
} // InitLCD

// RGB24 -> RGB565
uint16_t SSD1963_RGB565CONVERT(uint8_t red, uint8_t green, uint8_t blue)
{
  return (uint16_t)( (( red >> 3 ) << 11 ) | (( green >> 2 ) << 5  ) | ( blue >> 3 ));
} // RGB565CONVERT

//    ()
void SSD1963_SetOrient ( TOrientation orientation )
{
	orient = orientation;
} //  SetOrient

//    ()
TOrientation  SSD1963_GetOrient ( void )
{
	return orient;
} // SSD1963_GetOrient

//  
void SSD1963_SetFont ( TFont fontId )
{
	font = NULL;

	switch ( fontId )
	{
		case LucidaConsole12pt:
			font = &lucidaConsole12ptFontInfo;
			break;
	} // swich
} // SSD1963_SetFont

//   X
uint16_t SSD1963_GetMaxX ( void )
{
	if ( orient == Orientation_Portrait )
		return DISP_WIDTH - 1;

	// Album
	return DISP_HEIGHT - 1;
} // GetMaxX

//   Y
uint16_t SSD1963_GetMaxY ( void )
{
	if ( orient == Orientation_Portrait )
		return DISP_HEIGHT - 1;

	// Album
	return DISP_WIDTH - 1;
} // GetMaxY

//    
void SSD1963_FillScr ( uint16_t color )
{
	SSD1963_FillRect ( 0, 0, DISP_WIDTH, DISP_HEIGHT, color );
} // SSD1963_FillScr

//    
void SSD1963_FillRect ( uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2, uint16_t color )
{
	uint16_t x, y;

	for ( x = x1; x <= x2; x ++ )
		for ( y = y1; y <= y2; y ++ )
			SSD1963_PutPixel ( x, y, color );
} // SSD1963_FillRect

//    
void SSD1963_PutPixel ( uint16_t x, uint16_t y, uint16_t color )
{
	uint16_t x1 = x, y1 = y;
/*
	if ( orient == Orientation_Album )
	{ //     
		if ( y >= DISP_WIDTH || x >= DISP_HEIGHT )	//  
			return;

		x1 = y;
		y1 = DISP_HEIGHT - 1 - x;
	} // if
	else // 
		if ( x >= DISP_WIDTH || y >= DISP_HEIGHT )	//  
			return;
*/
	PutPixelInternal ( x1, y1, color );
} // SSD1963_PutPixel

//  
void SSD1963_drawLine ( int x1, int y1, int x2, int y2, uint16_t color)
{
  int deltaX, deltaY, signX, signY, error, error2;
  int dX, dY;
  int x, y;

  if ( x1 == x2 )
  { //  
    if ( y2 > y1 )
    {
      for ( y = y1; y < y2; y ++ )
    	  SSD1963_PutPixel ( x1, y, color );
    } // if
    else
    { // y1  y2
      for ( y = y2; y < y1; y ++ )
    	  SSD1963_PutPixel ( x1, y, color );
    } // if

    return;
  } // if

  if ( y1 == y2 )
  { // o 
    if ( x2 > x1 )
    {
      for ( x = x1; x < x2; x ++ )
    	  SSD1963_PutPixel ( x, y1, color );
    } // if
    else
    {
      for ( x = x2; x < x1; x ++ )
    	  SSD1963_PutPixel ( x, y1, color );
    } // else

    return;
  } // if

  dX = x2 - x1;
  dY = y2 - y1;
  deltaX = abs(dX);
  deltaY = abs(dY);
  signX = (x1 < x2) ? 1 : -1;
  signY = (y1 < y2) ? 1 : -1;
  error = deltaX - deltaY;

  SSD1963_PutPixel ( x2, y2, color );
  while ( x1 != x2 || y1 != y2 )
  {
	  SSD1963_PutPixel ( x1, y1, color );
	  error2 = error * 2;

    if ( error2 > -deltaY )
    {
      error -= deltaY;
      x1 += signX;
    } // if

    if ( error2 < deltaX )
    {
      error += deltaX;
      y1 += signY;
    } // if
  } // while
} // SSD1963_drawLine

//  
void SSD1963_DrawRect ( int x1, int y1, int x2, int y2, uint16_t color )
{
	//  
	SSD1963_drawLine ( x1, y1, x2, y1, color );
	//  
	SSD1963_drawLine ( x2, y1, x2, y2, color );
	//  
	SSD1963_drawLine ( x1, y2, x2, y2, color );
	//  
	SSD1963_drawLine ( x1, y1, x1, y2, color );
} // SSD1963_DrawRect

void SSD1963_SetCursor ( uint16_t Xpos, uint16_t Ypos )
{
	uint16_t x1 = Xpos, x2 = Xpos;
	uint16_t y1 = Ypos, y2 = Xpos;

	// set_column_address
	FSMC_LcdWriteCmd ( 0x002A );
	FSMC_LcdWriteData ( x1 >> 8 );
	FSMC_LcdWriteData ( x1&0x00ff );
	FSMC_LcdWriteData ( x2 >> 8 );
	FSMC_LcdWriteData ( x2 & 0x00ff );

	// set_page_addres
	FSMC_LcdWriteCmd ( 0x002b );
	FSMC_LcdWriteData ( y1 >> 8 );
	FSMC_LcdWriteData ( y1 & 0x00ff );
	FSMC_LcdWriteData ( y2 >> 8 );
	FSMC_LcdWriteData (y2 & 0x00ff );

	// write_memory_start
	FSMC_LcdWriteCmd ( 0x002c );
} // SSD1963_SetCursor

/*
void SSD1963_SetDisplayWindow ( uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2)
{
	uint16_t x1_, y1_, x2_, y2_;

	//    
	if ( orient == Orientation_Portrait )
	{ // 
		//  
		if ( x1 >= DISP_WIDTH || y1 >= DISP_HEIGHT || x2 >= DISP_WIDTH || y2 >= DISP_HEIGHT ||
				x1 > x2 || y1 > y2 )
			return;
		x1_ = x1;
		y1_ = y1;
		x2_ = x2;
		y2_ = y2;
	} // else
	else
	{ //  -  
//		if ( y >= DISP_WIDTH || x >= DISP_HEIGHT )	//  
//			return;

		x1_ = y1;
		y1_ = DISP_HEIGHT - 1 - x2;
		x2_ = y2;
		y2_ = DISP_HEIGHT - 1 - x1;
	} // if

	// Horizontal RAM address position
	// End   -  
	// Start -  
//	ArduinoLcdWriteReg ( 0x44, ( x2_ << 8 ) | x1_ );
//	ArduinoLcdWriteReg ( 0x45, y1_ );	// Vertical RAM address start position
//	ArduinoLcdWriteReg ( 0x46, y2_ );	// Vertical RAM address end position

	// 150 ms
//	delay_ms(15000);
	delay_ms(1);

	// SetCursor
	SSD1963_SetCursor ( x1_, y1_ );
} // SSD1963_SetDisplayWindow

//  BMP
void SSD1963_WriteBMP ( uint8_t x1, uint16_t y1, uint16_t Width, uint8_t Height, void *bitmap,
		int rotate, bool isVertical )
{
} // SSD1963_WriteBMP
*/

//////////////////////////////////////
//   

//    ( )
uint8_t SSD1963_DrawChar ( uint16_t x, uint16_t y, uint8_t c, uint16_t textColor,
		uint16_t backColor, uint8_t isTransparent )
{
 uint8_t i, j;
 uint16_t mask, b, pixX, pixY;
 uint8_t width, height, charIdx;
 const uint8_t *ptr;
 uint8_t FirstChar, LastChar;

 if ( !font )
  return 0;                                                //       

 FirstChar = font->FirstChar;
 LastChar = font->LastChar;

 //           
 if(c < FirstChar || c > LastChar)
  return 0;

 //     
 charIdx = c - font->FirstChar;

 //  
 width = font->FontTable[charIdx].width;

 //  
 height = font->Height;

 //     
 ptr = font -> FontBitmaps;
 b = font -> FontTable[charIdx].start;
 ptr += b;

 // 1-  
 pixY = y;

 //    
 for ( i = 0; i < height; i ++ )
 {
   //     
   pixX = x;

   mask = 0x8000;
   //    
   b = *ptr;       //  
   b <<= 8;
   ptr ++;
   j = *ptr;    //  
   ptr ++;
   b |= j;

   //    
   for ( j = 0; j < width; j ++ )
   {
	   if ( (b&mask) !=0 )
		   SSD1963_PutPixel ( pixX, pixY, textColor ); //  
	   else //  
	   {
		   if ( isTransparent != 0 )       //  -  
			   SSD1963_PutPixel ( pixX, pixY, backColor );
	   } // else

	   // 
	   pixX ++;
	   mask >>= 1;
   } // for - 

   //  
   pixY ++;
 } // for - 

 return width;
} // SSD1963_DrawChar

//   
uint8_t SSD1963_GetCharHight ( void )
{
  return font -> Height;
} // SSD1963_GetCharHight

//   new
void SSD1963_DrawString ( char* s, uint16_t x, uint16_t y, uint16_t color, uint16_t backColor, uint8_t isTransparent )
{
	int i, len = strlen(s);

	for ( i = 0; i < len; i ++ )
		x += SSD1963_DrawChar ( x, y, s[i], color, backColor, isTransparent );
} // SSD1963_DrawString

//
//////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////////////
//     

//     (,    )
static void PutPixelInternal ( uint16_t x, uint16_t y, uint16_t color )
{
	SSD1963_SetCursor ( x, y );

	FSMC_LcdWriteData ( color );
} // PutPixelInternal

//
/////////////////////////////////////////////////////////////////////////////////////////////
